
--
--  Custom Flight Map
--    by Tuhljin
--


CustomFlightMap = {}

local ICONS = {
	"Interface\\TaxiFrame\\UI-Taxi-Icon-Red",
	"Interface\\AddOns\\CustomFlightMap\\Textures\\TaxiLightGreen";
	"Interface\\AddOns\\CustomFlightMap\\Textures\\TaxiLightBlue";
	"Interface\\AddOns\\CustomFlightMap\\Textures\\TaxiBlue";
	"Interface\\AddOns\\CustomFlightMap\\Textures\\TaxiPurple";
	"Interface\\TaxiFrame\\UI-Taxi-Icon-Gray";
	"Interface\\AddOns\\CustomFlightMap\\Textures\\AllianceTower";
	"Interface\\AddOns\\CustomFlightMap\\Textures\\HordeTower";
	"Interface\\AddOns\\CustomFlightMap\\Textures\\NeutralTower";
	false;
}
local ICONS_HL = {
	["Interface\\AddOns\\CustomFlightMap\\Textures\\AllianceTower"] = "Interface\\AddOns\\CustomFlightMap\\Textures\\TowerHighlight";
	["Interface\\AddOns\\CustomFlightMap\\Textures\\HordeTower"] = "Interface\\AddOns\\CustomFlightMap\\Textures\\TowerHighlight";
	["Interface\\AddOns\\CustomFlightMap\\Textures\\NeutralTower"] = "Interface\\AddOns\\CustomFlightMap\\Textures\\TowerHighlight";
	["Interface\\AddOns\\CustomFlightMap\\Textures\\TowerGreen"] = "Interface\\AddOns\\CustomFlightMap\\Textures\\TowerHighlight";
}
local ICONS_CURRENT = {
	["Interface\\AddOns\\CustomFlightMap\\Textures\\AllianceTower"] = "Interface\\AddOns\\CustomFlightMap\\Textures\\TowerGreen";
	["Interface\\AddOns\\CustomFlightMap\\Textures\\HordeTower"] = "Interface\\AddOns\\CustomFlightMap\\Textures\\TowerGreen";
	["Interface\\AddOns\\CustomFlightMap\\Textures\\NeutralTower"] = "Interface\\AddOns\\CustomFlightMap\\Textures\\TowerGreen";
}
local ICON_DEFAULT = "Interface\\TaxiFrame\\UI-Taxi-Icon-White"
local ICON_DEFAULT_HL = "Interface\\TaxiFrame\\UI-Taxi-Icon-Highlight"

local SIZES = { (6/5), (7/5), (4/5), 1 }  -- Multipliers, not the actual size
local SIZE_DEFAULT  -- Base (actual) size
local SIZE_DEFAULT_HL  -- Base size for highlight

local NODES_ICON, NODES_SIZE

local regclick, orig_OnClick, unhighlighted = 0

local function fixUnhighlighted()
	if (unhighlighted) then
		unhighlighted:Show()
		unhighlighted = nil
	end
end

local function setTexture(button, tex)
	button:SetNormalTexture(tex)
	button:SetHighlightTexture(ICONS_HL[tex] or ICON_DEFAULT_HL)
	local hl = button:GetHighlightTexture()
	hl:SetBlendMode(tex == ICON_DEFAULT and "ALPHAKEY" or "ADD")
	hl:SetAlpha(tex == ICON_DEFAULT and 1 or 0.5)
end

local function nodeOnClick(self, btn, ...)
	if (btn == "RightButton") then
		if (CustomFlightMap_Settings.Locked) then  return;  end
		local id = self:GetID()
		local name = TaxiNodeName(id)

		if (IsShiftKeyDown()) then
			local s
			if (IsControlKeyDown()) then
				s = 1
			elseif (NODES_SIZE[name]) then
				local prev = NODES_SIZE[name]
				for i = 1, (#SIZES - 1) do  -- Find the next place in the sizes list (if any)
					if (SIZES[i] == prev) then
						s = SIZES[i+1]  -- We used (#SIZES - 1) so adding 1 will never give a number greater than the size of the list.
						break;
					end
				end
			end
			s = s or SIZES[1]
			if (s == 1) then  NODES_SIZE[name] = nil;  else  NODES_SIZE[name] = s;  end
			self:SetSize(s * SIZE_DEFAULT, s * SIZE_DEFAULT)
			self:GetHighlightTexture():SetSize(s * SIZE_DEFAULT_HL, s * SIZE_DEFAULT_HL)
			return;
		end

		local tex
		if (IsControlKeyDown()) then
			tex = false
		else
			if (NODES_ICON[name]) then
				local prev = NODES_ICON[name]
				for i = 1, (#ICONS - 1) do  -- Find the next place in the icons list (if any)
					if (ICONS[i] == prev) then
						tex = ICONS[i+1]  -- We used (#ICONS - 1) so adding 1 will never give a number greater than the size of the list.
						break;
					end
				end
			end
			tex = tex or tex == nil and ICONS[1] or false  -- Default to the first icon in the list.
		end
		if (tex) then  NODES_ICON[name] = tex;  else  NODES_ICON[name] = nil;  end
		setTexture(self, tex or TaxiNodeGetType(id)=="CURRENT" and "Interface\\TaxiFrame\\UI-Taxi-Icon-Green" or ICON_DEFAULT)
		-- Hide the highlight temporarily so we can see the new normal texture we set:
		if (unhighlighted) then  unhighlighted:Show();  end
		unhighlighted = self:GetHighlightTexture()
		unhighlighted:Hide()
	else
		return orig_OnClick(self, btn, ...)
	end
end

TaxiFrame:HookScript("OnEvent", function(self, event)
	if (event ~= "TAXIMAP_OPENED" or not self:IsShown()) then  return;  end
	local num_nodes = NumTaxiNodes()
	local button, name, s
	for i = 1, num_nodes do
		button = _G["TaxiButton"..i]
		if (i > regclick) then
			regclick = i
			SIZE_DEFAULT = SIZE_DEFAULT or button:GetSize()
			SIZE_DEFAULT_HL = SIZE_DEFAULT_HL or button:GetHighlightTexture():GetSize()
			button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
			orig_OnClick = orig_OnClick or button:GetScript("OnClick")
			button:SetScript("OnClick", nodeOnClick)
			button:HookScript("OnEnter", fixUnhighlighted)
		end
		if (button:IsShown()) then
			name = TaxiNodeName(i)

			if (not CustomFlightMap_Settings.Hidden and NODES_ICON[name]) then
				s = NODES_ICON[name]
				if (TaxiNodeGetType(i) == "CURRENT") then
					if (ICONS_CURRENT[s]) then  setTexture(button, ICONS_CURRENT[s]);  end
					-- Note that we don't change the texture if this is the CURRENT node and there's nothing in ICONS_CURRENT for this texture. (The default green node is kept in place.)
				else
					setTexture(button, s)
				end
			end

			if (not CustomFlightMap_Settings.Hidden and NODES_SIZE[name]) then
				s = NODES_SIZE[name] * SIZE_DEFAULT
				button:SetSize(s,s)
				s = NODES_SIZE[name] * SIZE_DEFAULT_HL
				button:GetHighlightTexture():SetSize(s,s)
			elseif (button:GetSize() ~= SIZE_DEFAULT) then  -- Since the default interface doesn't adjust size, we need to set it to default here if it was changed:
				button:SetSize(SIZE_DEFAULT,SIZE_DEFAULT)
				button:GetHighlightTexture():SetSize(SIZE_DEFAULT_HL,SIZE_DEFAULT_HL)
			end
		end
	end
end);


CustomFlightMap.MainFrame = CreateFrame("Frame")
CustomFlightMap.MainFrame:Hide()
CustomFlightMap.MainFrame:RegisterEvent("ADDON_LOADED")
CustomFlightMap.MainFrame:SetScript("OnEvent", function(self, event, arg1, ...)
	if (arg1 == "CustomFlightMap") then
		self:UnregisterEvent("ADDON_LOADED")
		CustomFlightMap_Settings = CustomFlightMap_Settings or {}
		CustomFlightMap_Settings.Version = GetAddOnMetadata("CustomFlightMap", "Version")
		CustomFlightMap_Settings.NODES_ICON = CustomFlightMap_Settings.NODES_ICON or {}
		CustomFlightMap_Settings.NODES_SIZE = CustomFlightMap_Settings.NODES_SIZE or {}
		NODES_ICON, NODES_SIZE = CustomFlightMap_Settings.NODES_ICON, CustomFlightMap_Settings.NODES_SIZE
	end
end);

SLASH_CustomFlightMap1 = "/cfm";
SlashCmdList["CustomFlightMap"] = function(msg, self)
	msg = strlower(msg)
	if (msg == "reset") then
		local num_nodes = NumTaxiNodes()
		if (num_nodes == 0) then
			print("|cffffc133Custom Flight Map: You must be at a flight master to do that.")
		else
			local name
			for i = 1, num_nodes do
				name = TaxiNodeName(i)
				NODES_ICON[name], NODES_SIZE[name] = nil, nil
			end
			print("|cffffc133Custom Flight Map: The displayed nodes' changes have been reset.")
			TaxiFrame:GetScript("OnEvent")(TaxiFrame, "TAXIMAP_OPENED")
		end
	elseif (msg == "resetall") then
		wipe(CustomFlightMap_Settings.NODES_ICON)
		wipe(CustomFlightMap_Settings.NODES_SIZE)
		if (TaxiFrame:IsShown()) then  TaxiFrame:GetScript("OnEvent")(TaxiFrame, "TAXIMAP_OPENED");  end
		print("|cffffc133Custom Flight Map: All node changes have been reset.")
	elseif (msg == "lock") then
		CustomFlightMap_Settings.Locked = true
		print("|cffffc133Custom Flight Map: Changes locked.")
	elseif (msg == "unlock") then
		CustomFlightMap_Settings.Locked = nil
		CustomFlightMap_Settings.Hidden = nil
		if (TaxiFrame:IsShown()) then  TaxiFrame:GetScript("OnEvent")(TaxiFrame, "TAXIMAP_OPENED");  end
		print("|cffffc133Custom Flight Map: Changes unlocked and shown.")
	elseif (msg == "hide") then
		CustomFlightMap_Settings.Locked = true
		CustomFlightMap_Settings.Hidden = true
		if (TaxiFrame:IsShown()) then  TaxiFrame:GetScript("OnEvent")(TaxiFrame, "TAXIMAP_OPENED");  end
		print("|cffffc133Custom Flight Map: Changes locked and hidden.")
	elseif (msg == "show") then
		CustomFlightMap_Settings.Hidden = nil
		if (TaxiFrame:IsShown()) then  TaxiFrame:GetScript("OnEvent")(TaxiFrame, "TAXIMAP_OPENED");  end
		print("|cffffc133Custom Flight Map: Changes shown.")
	else
		print("|cffffc133Custom Flight Map v"..GetAddOnMetadata("CustomFlightMap", "Version").." commands:")
		print(" /cfm lock      |cffffc133Prevent node changes.")
		print(" /cfm unlock   |cffffc133Allow and show node changes.")
		print(" /cfm hide      |cffffc133Hide and prevent node changes.")
		print(" /cfm show     |cffffc133Show node changes.")
		print(" /cfm reset     |cffffc133Reset the currently displayed nodes' changes.")
		print(" /cfm resetall  |cffffc133Reset ALL node changes.")
		return;
	end
end

-- /dump TaxiNodeName( GetMouseFocus():GetID() )
-- /dump CustomFlightMap_Settings
